# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import setuptools

long_description = open("README.md", "r", encoding="utf-8").read()

setuptools.setup(
    name="GeoSnipe",
    version="1.2.5",
    author="Geographs",
    maintainer="Geographs",
    author_email="87452561+Geographs@users.noreply.github.com",
    description="A pythonic Minecraft username sniper based on asyncio.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Geographs/GeoSnipe",
    download_url="https://github.com/Geographs/GeoSnipe/releases",
    project_urls={
        "GitHub": "https://github.com/Geographs/GeoSnipe",
        "Documentation": "https://docs.geographs.pro",
        "Bug Tracker": "https://github.com/Geographs/GeoSnipe/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=["geosnipe", "geosnipe.auth"],
    python_requires=">=3.9",
    license="MIT",
    entry_points={"console_scripts": ["geosnipe=geosnipe.__main__:main"]},
    install_requires=["msmcauth==0.0.3", "paramiko>=2.7.2"]
)
